@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 and 2006       *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -28,9 +28,6 @@
 #include <string.h>
 #include "liberty.h"
 
-static SGFTree *aftermath_sgftree;
-
-
 static int do_aftermath_genmove(int color,
 				int under_control[BOARDMAX],
 				int do_capture_dead_stones);
@@ -53,7 +50,7 @@ all_own_neighbors_inessential(int pos, int color)
 
 /* External interface to do_aftermath_genmove().
  *
- * If the suggested move turn out not to be allowed we just return
+ * If the suggested move turns out not to be allowed we just return
  * pass. This is not ideal but also not a big deal. If
  * do_aftermath_genmove() is ever redesigned that would be a good time
  * to integrate allowed_moves.
@@ -671,6 +668,7 @@ do_aftermath_genmove(int color,
 	continue;
       if (board[pos2] == other 
 	  && dragon[pos2].status != ALIVE
+	  && dragon[pos2].status != UNKNOWN
 	  && (do_capture_dead_stones 
 	      || worm[pos2].unconditional_status != DEAD)
 	  && DRAGON2(pos2).safety != INESSENTIAL) {
@@ -752,9 +750,10 @@ do_aftermath_genmove(int color,
     /* Make sure that the potential move really isn't a self
      * atari. In the case of a move found after backfilling this
      * could happen (because the backfilling moves happened to
-     * capture some stones).
+     * capture some stones). The position of the move may even be
+     * occupied.
      */
-    if (!self_atari_ok && is_self_atari(move, color))
+    if (!self_atari_ok && (board[move] != EMPTY || is_self_atari(move, color)))
       continue;
     
     /* Consult the owl code to determine whether the considered move
@@ -842,7 +841,7 @@ reduced_genmove(int color)
   reset_engine();
 
   /* Find out information about the worms and dragons. */
-  examine_position(EXAMINE_ALL);
+  examine_position(EXAMINE_ALL, 1);
 
   /* The score will be used to determine when we are safely
    * ahead. So we want the most conservative score.
@@ -885,7 +884,8 @@ reduced_genmove(int color)
 
 /* Preliminary function for playing through the aftermath. */
 static void
-do_play_aftermath(int color, struct aftermath_data *a)
+do_play_aftermath(int color, struct aftermath_data *a,
+		  SGFTree *aftermath_sgftree)
 {
   int move;
   int pass = 0;
@@ -935,7 +935,7 @@ do_play_aftermath(int color, struct aftermath_data *a)
 static struct aftermath_data aftermath;
 
 static void
-play_aftermath(int color)
+play_aftermath(int color, SGFTree *aftermath_sgftree)
 {
   int pos;
   struct board_state saved_board;
@@ -973,7 +973,7 @@ play_aftermath(int color)
   a->black_area = 0;
   
   store_board(&saved_board);
-  do_play_aftermath(color, a);
+  do_play_aftermath(color, a, aftermath_sgftree);
   restore_board(&saved_board);
   
   for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
@@ -1033,15 +1033,15 @@ play_aftermath(int color)
 }
 
 float
-aftermath_compute_score(int color, float komi, SGFTree *tree)
+aftermath_compute_score(int color, SGFTree *tree)
 {
   struct aftermath_data *a = &aftermath;
-  aftermath_sgftree = tree;
-  play_aftermath(color);
+  play_aftermath(color, tree);
   if (chinese_rules)
     return (a->white_area
 	    - a->black_area
-	    + komi);
+	    + komi
+	    + handicap);
   else
     return (a->white_territory
 	    + a->black_captured
@@ -1060,8 +1060,7 @@ enum dragon_status
 aftermath_final_status(int color, int pos)
 {
   ASSERT_ON_BOARD1(pos);
-  aftermath_sgftree = NULL;
-  play_aftermath(color);
+  play_aftermath(color, NULL);
   return aftermath.final_status[pos];
 }
 
