@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 and 2006       *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -148,13 +148,6 @@ make_dragons(int stop_before_owl)
     if (IS_STONE(board[str]) && dragon[str].origin == str)
       DRAGON2(str).escape_route = compute_escape(str, 0);
 
-  /* Update the segmentation of the initial influence before we
-   * compute the surrounding moyo sizes. The reason for this is that
-   * now the eyespace inhibition found by find_cuts() can be taken
-   * into account.
-   */
-  resegment_initial_influence();
-
   /* Set dragon weaknesses according to initial_influence. */
   compute_refined_dragon_weaknesses();
   for (d = 0; d < number_of_dragons; d++)
@@ -163,19 +156,15 @@ make_dragons(int stop_before_owl)
   /* Determine status: ALIVE, DEAD, CRITICAL or UNKNOWN */
   for (str = BOARDMIN; str < BOARDMAX; str++)
     if (ON_BOARD(str))
-      if (dragon[str].origin == str && board[str]) {
+      if (dragon[str].origin == str && board[str])
 	dragon[str].crude_status = compute_crude_status(str);
-      }
   
   /* We must update the dragon status at every intersection before we
    * call the owl code. This updates all fields.
    */
   for (str = BOARDMIN; str < BOARDMAX; str++)
-    if (ON_BOARD(str)) {
-      struct dragon_data *dd = &(dragon[str]);
-      
-      dragon[str] = dragon[dd->origin];
-    }
+    if (ON_BOARD(str) && board[str] != EMPTY)
+      dragon[str] = dragon[dragon[str].origin];
   
   find_neighbor_dragons();
 
@@ -226,9 +215,12 @@ make_dragons(int stop_before_owl)
 	int acode = 0;
 	int dcode = 0;
 	int kworm = NO_MOVE;
+	int owl_nodes_before = get_owl_node_counter();
 	start_timer(3);
 	acode = owl_attack(str, &attack_point, 
 			   &DRAGON2(str).owl_attack_certain, &kworm);
+	DRAGON2(str).owl_attack_node_count
+	  = get_owl_node_counter() - owl_nodes_before;
 	if (acode != 0) {
 	  DRAGON2(str).owl_attack_point = attack_point;
 	  DRAGON2(str).owl_attack_code = acode;
@@ -299,40 +291,24 @@ make_dragons(int stop_before_owl)
   time_report(2, "  owl reading", NO_MOVE, 1.0);
   
   /* Compute the status to be used by the matcher. We most trust the
-   * owl status, if it is available.
+   * owl status, if it is available. If it's not we assume that we are
+   * already confident that the dragon is alive, regardless of
+   * crude_status.
    */
   for (str = BOARDMIN; str < BOARDMAX; str++)
-    if (ON_BOARD(str)) {
-      if (IS_STONE(board[str])) {
-	if (DRAGON2(str).owl_status != UNCHECKED)
-	  dragon[str].status = DRAGON2(str).owl_status;
-	else if (dragon[str].crude_status == DEAD 
-		 || dragon[str].crude_status == CRITICAL) {
-	  /* If a dragon has sufficient escape potential or
-	   * surrounding moyo to stop the owl code from being run, the
-	   * status should be no worse than UNKNOWN,
-	   * regardless what the static life and death analysis
-	   * guesses.
-	   */
-	  dragon[str].status = UNKNOWN;
-	}
-	else {
-	  /* And if the static life and death analysis said UNKNOWN,
-           * we are most likely ALIVE.
-	   */
-	  dragon[str].status = ALIVE;
-	}
-      }
+    if (IS_STONE(board[str])) {
+      if (DRAGON2(str).owl_status != UNCHECKED)
+	dragon[str].status = DRAGON2(str).owl_status;
+      else
+	dragon[str].status = ALIVE;
     }
 
   /* The dragon data is now correct at the origin of each dragon but
    * we need to copy it to every vertex.  
    */
   for (str = BOARDMIN; str < BOARDMAX; str++)
-    if (ON_BOARD(str)) {
-      struct dragon_data *dd = &(dragon[str]);
-      dragon[str] = dragon[dd->origin];
-    }
+    if (ON_BOARD(str) && board[str] != EMPTY)
+      dragon[str] = dragon[dragon[str].origin];
 
   /* Owl threats. */
   for (str = BOARDMIN; str < BOARDMAX; str++)
@@ -354,7 +330,7 @@ make_dragons(int stop_before_owl)
 	int dcode = DRAGON2(str).owl_defense_code;
 	int defense_point, second_defense_point;
 	
-	if (level >= 8
+	if (get_level() >= 8
 	    && !disable_threat_computation
 	    && (owl_threats 
 		|| thrashing_stone[str])) {
@@ -387,10 +363,8 @@ make_dragons(int stop_before_owl)
    * but we need to copy it to every vertex.  
    */
   for (str = BOARDMIN; str < BOARDMAX; str++)
-    if (ON_BOARD(str)) {
-      struct dragon_data *dd = &(dragon[str]);
-      dragon[str] = dragon[dd->origin];
-    }
+    if (ON_BOARD(str) && board[str] != EMPTY)
+      dragon[str] = dragon[dragon[str].origin];
 
   time_report(2, "  owl threats ", NO_MOVE, 1.0);
   
@@ -752,7 +726,9 @@ initialize_supplementary_dragon_data(void)
     set_eyevalue(&dragon2[d].genus, 0, 0, 0, 0);
 
     dragon2[d].semeais                  = 0;
+    dragon2[d].semeai_defense_code	= 0;
     dragon2[d].semeai_defense_point	= NO_MOVE;
+    dragon2[d].semeai_attack_code	= 0;
     dragon2[d].semeai_attack_point	= NO_MOVE;
     dragon2[d].owl_attack_point         = NO_MOVE;
     dragon2[d].owl_attack_code          = 0;
@@ -1444,8 +1420,8 @@ connected_to_eye(int pos, int str, int color, int eye_color,
  */
 static void
 connected_to_eye_recurse(int pos, int str, int color, int eye_color,
-			 struct eye_data *eye, signed char *mx, signed char *me,
-			 int *halfeyes)
+			 struct eye_data *eye, signed char *mx,
+			 signed char *me, int *halfeyes)
 {
   int liberties;
   int libs[MAXLIBS];
@@ -1599,9 +1575,13 @@ show_dragons(void)
       }
       if (dd->status == CRITICAL && d2->semeais) {
 	if (d2->semeai_defense_point)
-	  gprintf("... semeai defense move at %1m\n", d2->semeai_defense_point);
+	  gprintf("... semeai defense move at %1m, result code %s\n",
+		  d2->semeai_defense_point,
+		  result_to_string(d2->semeai_defense_code));
 	if (d2->semeai_attack_point)
-	  gprintf("... semeai attack move at %1m\n", d2->semeai_attack_point);
+	  gprintf("... semeai attack move at %1m, result code %s\n",
+		  d2->semeai_attack_point,
+		  result_to_string(d2->semeai_attack_code));
       }
       gprintf("... neighbors");
       for (k = 0; k < d2->neighbors; k++) {
@@ -1670,7 +1650,7 @@ join_new_dragons(int d1, int d2)
   ASSERT1(board[d1] == board[d2], d1);
   ASSERT1(IS_STONE(board[d1]), d1);
 
-  /* Don't bother to do anything fance with dragon origins. */
+  /* Don't bother to do anything fancy with dragon origins. */
   for (pos = BOARDMIN; pos < BOARDMAX; pos++)
     if (ON_BOARD(pos) && new_dragon_origins[pos] == d2)
       new_dragon_origins[pos] = d1;
@@ -2065,14 +2045,11 @@ compute_surrounding_moyo_sizes(const struct influence_data *q)
 {
   int pos;
   int d;
-
   int k;
-  int moyo_color[BOARDMAX];
-  float territory_value[BOARDMAX];
+  int moyo_color;
   float moyo_sizes[BOARDMAX];
   float moyo_values[BOARDMAX];
     
-  influence_get_moyo_data(q, moyo_color, territory_value);
 
   for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
     moyo_sizes[pos] = 0.0;
@@ -2082,28 +2059,29 @@ compute_surrounding_moyo_sizes(const struct influence_data *q)
   for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
     if (!ON_BOARD(pos))
       continue;
+    moyo_color = whose_moyo_restricted(q, pos);
     
-    if (moyo_color[pos] == board[pos])
+    if (moyo_color == board[pos])
       continue;
     
-    if (moyo_color[pos] == WHITE) {
+    if (moyo_color == WHITE) {
       for (k = 0; k < number_close_white_worms[pos]; k++) {
 	int w = close_white_worms[pos][k];
 	int dr = dragon[w].origin;
 	
 	moyo_sizes[dr] += 1.0 / number_close_white_worms[pos];
-	moyo_values[dr] += (gg_min(territory_value[pos], 1.0)
+	moyo_values[dr] += (gg_min(influence_territory(q, pos, WHITE), 1.0)
 			    / number_close_white_worms[pos]);
       }
     }
     
-    if (moyo_color[pos] == BLACK) {
+    if (moyo_color == BLACK) {
       for (k = 0; k < number_close_black_worms[pos]; k++) {
 	int w = close_black_worms[pos][k];
 	int dr = dragon[w].origin;
 	
 	moyo_sizes[dr] += 1.0 / number_close_black_worms[pos];
-	moyo_values[dr] += (gg_min(territory_value[pos], 1.0)
+	moyo_values[dr] += (gg_min(influence_territory(q, pos, BLACK), 1.0)
 			    / number_close_black_worms[pos]);
       }
     }
@@ -2245,6 +2223,62 @@ compute_refined_dragon_weaknesses()
     dragon2[d].weakness = compute_dragon_weakness_value(d);
 }
 
+/* The strategic size is the effective size, plus a bonus for all weak
+ * neighbouring dragons of the opponent.
+ */
+void
+compute_strategic_sizes()
+{
+  float *bonus = calloc(number_of_dragons, sizeof(float));
+  int d;
+  int k;
+
+  for (d = 0; d < number_of_dragons; d++) {
+    /* Compute bonus for all neighbors of dragon (d). The total bonus for
+     * all neighbors is effective_size(d) * weakness(d), and it is given
+     * to a neighbor d2 proportionally to the value of
+     * effective_size(d2) * weakness(d2).
+     */
+    float sum = 0.0;
+    if (dragon2[d].safety == INESSENTIAL)
+      continue;
+    for (k = 0; k < dragon2[d].neighbors; k++) {
+      int d2 = dragon2[d].adjacent[k];
+      if (board[dragon2[d2].origin] == OTHER_COLOR(board[dragon2[d].origin])
+	  && dragon2[d2].safety != INESSENTIAL)
+	sum += DRAGON(d2).effective_size * dragon2[d2].weakness;
+    }
+    if (sum == 0.0)
+      continue;
+    for (k = 0; k < dragon2[d].neighbors; k++) {
+      int d2 = dragon2[d].adjacent[k];
+      if (board[dragon2[d2].origin] == OTHER_COLOR(board[dragon2[d].origin])
+	  && dragon2[d2].safety != INESSENTIAL) {
+	bonus[d2] += ((DRAGON(d2).effective_size * dragon2[d2].weakness) / sum)
+		     * DRAGON(d).effective_size * dragon2[d].weakness;
+	if (0)
+	  gprintf("Dragon %1m receives %f effective size bonus from %1m.\n",
+		  dragon2[d2].origin, 
+		  ((DRAGON(d2).effective_size * dragon2[d2].weakness) / sum)
+		  * DRAGON(d).effective_size * dragon2[d].weakness,
+		  dragon2[d].origin);
+      }
+    }
+  }
+
+  for (d = 0; d < number_of_dragons; d++) {
+    if (0)
+      gprintf("Dragon %1m gets effective size bonus of %f.\n",
+	      dragon2[d].origin, bonus[d]);
+    /* We cap strategic size at 3 * effective_size. (This is ad hoc.) */
+    dragon2[d].strategic_size = gg_min(bonus[d] + DRAGON(d).effective_size,
+				       3 * DRAGON(d).effective_size);
+  }
+
+  free(bonus);
+}
+
+
 /* 
  * Test whether two dragons are the same. Used by autohelpers and elsewhere.
  */
@@ -2420,7 +2454,7 @@ ascii_report_dragon(char *string)
 void
 report_dragon(FILE *outfile, int pos)
 {
-  int ii;
+  int w;
   int k;
   struct dragon_data *d = &(dragon[pos]);
   struct dragon_data2 *d2 = &(dragon2[d->id]);
@@ -2439,6 +2473,7 @@ report_dragon(FILE *outfile, int pos)
   gfprintf(outfile, "origin                  %1m\n", d->origin);
   gfprintf(outfile, "size                    %d\n", d->size);
   gfprintf(outfile, "effective_size          %f\n", d->effective_size);
+  gfprintf(outfile, "strategic_size          %f\n", d2->strategic_size);
   gfprintf(outfile, "genus                   %s\n",
 	   eyevalue_to_string(&d2->genus));
   gfprintf(outfile, "heye                    %1m\n", d2->heye);
@@ -2480,20 +2515,19 @@ report_dragon(FILE *outfile, int pos)
   gfprintf(outfile, "owl_attack_kworm        %1m\n", d2->owl_attack_kworm);
   gfprintf(outfile, "owl_defense_kworm       %1m\n", d2->owl_defense_kworm);
   gfprintf(outfile, "semeais                 %d\n", d2->semeais);
+  gfprintf(outfile, "semeai_defense_code     %d\n", d2->semeai_defense_code);
   gfprintf(outfile, "semeai_defense_point    %1m\n", d2->semeai_defense_point);
   gfprintf(outfile, "semeai_defense_certain  %d\n",
 	   d2->semeai_defense_certain);
   gfprintf(outfile, "semeai_defense_target   %1m\n",
       	   d2->semeai_defense_target);
+  gfprintf(outfile, "semeai_attack_code      %d\n", d2->semeai_attack_code);
   gfprintf(outfile, "semeai_attack_point     %1m\n", d2->semeai_attack_point);
   gfprintf(outfile, "semeai_attack_certain   %d\n", d2->semeai_attack_certain);
   gfprintf(outfile, "semeai_attack_target    %1m\n", d2->semeai_attack_target);
   gfprintf(outfile, "strings                 ");
-  for (ii = BOARDMIN; ii < BOARDMAX; ii++)
-    if (ON_BOARD(ii)
-	&& worm[ii].origin == ii
-	&& is_same_dragon(ii, pos))
-	gfprintf(outfile, "%1m ", ii);
+  for (w = first_worm_in_dragon(pos); w != NO_MOVE; w = next_worm_in_dragon(w))
+    gfprintf(outfile, "%1m ", w);
   gfprintf(outfile, "\n");
 }
 
