@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 and 2006       *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -36,6 +36,13 @@
  */
 #define NEEDS_UPDATE(x) (x != position_number ? (x = position_number, 1) : 0)
 
+/* Mark a limited search area. If limit_search != 1, genmove
+ * will only return moves within the area marked by the array
+ * search_mask.
+ */
+static int limit_search = 0;
+static int search_mask[BOARDMAX];
+
 static int do_genmove(int color, float pure_threat_value,
 		      int allowed_moves[BOARDMAX], float *value, int *resign);
 
@@ -54,7 +61,7 @@ static int revise_thrashing_dragon(int color, float our_score,
 static void break_mirror_go(int color);
 static int find_mirror_move(int *move, int color);
 static int should_resign(int color, float optimistic_score, int move);
-static void compute_scores(void);
+static void compute_scores(int use_chinese_rules);
 
 
 /* Reset some things in the engine. 
@@ -91,7 +98,7 @@ reset_engine()
   clear_break_in_list();
 
   /* Set up depth values (see comments there for details). */
-  set_depth_values(level, 0);
+  set_depth_values(get_level(), 0);
 }
 
 /*
@@ -102,10 +109,13 @@ reset_engine()
  * The parameter how_much tells us how much of the work we have to do.
  * For move generation we have to do it all.  For debugging we can 
  * sometimes stop a little earlier.
+ *
+ * aftermath_play indicates we are in aftermath playout phase. It's only
+ * effect is to always use chinese rules for the score estimates.
  */
 
 void
-examine_position(int how_much)
+examine_position(int how_much, int aftermath_play)
 {
   int save_verbose = verbose;
 
@@ -148,7 +158,7 @@ examine_position(int how_much)
     
     if (NEEDS_UPDATE(dragons_examined)) {
       make_dragons(0);
-      compute_scores();
+      compute_scores(chinese_rules || aftermath_play);
       /* We have automatically done a partial dragon analysis as well. */
       dragons_examined_without_owl = position_number;
     }
@@ -162,7 +172,7 @@ examine_position(int how_much)
 	   || how_much == EXAMINE_DRAGONS
 	   || how_much == EXAMINE_ALL) {
     initialize_dragon_data();
-    compute_scores();
+    compute_scores(chinese_rules || aftermath_play);
     verbose = save_verbose;
     gg_assert(test_gray_border() < 0);
     return;
@@ -180,6 +190,7 @@ examine_position(int how_much)
 
   if (NEEDS_UPDATE(dragons_refinedly_examined)) {
     compute_refined_dragon_weaknesses();
+    compute_strategic_sizes();
   }
   if (how_much == FULL_EXAMINE_DRAGONS) {
     gg_assert(test_gray_border() < 0);
@@ -209,7 +220,7 @@ silent_examine_position(int how_much)
   debug = 0;
   printmoyo = 0;
   
-  examine_position(how_much);
+  examine_position(how_much, 0);
 
   verbose = save_verbose;
   sgf_dumptree = save_sgf_dumptree;
@@ -234,14 +245,17 @@ genmove(int color, float *value, int *resign)
 
 #if ORACLE
   if (metamachine) {
-    move = metamachine_genmove(color, value);
+    move = metamachine_genmove(color, value, limit_search);
     gg_assert(stackp == 0);
     if (move != PASS_MOVE)
       return move;
   }
 #endif
 
-  move = do_genmove(color, 0.4, NULL, value, resign);
+  if (limit_search)
+    move = do_genmove(color, 0.4, search_mask, value, resign);
+  else
+    move = do_genmove(color, 0.4, NULL, value, resign);
   gg_assert(move == PASS_MOVE || ON_BOARD(move));
 
   return move;
@@ -285,9 +299,6 @@ collect_move_reasons(int color)
  * NULL any move is allowed. Pass is always allowed and will be chosen
  * if the move generation doesn't like any of the allowed moves (or
  * overlooks them).
- *
- * Resignation is indicated by a negatively valued move (which is not
- * a pass)
  */
   
 static int
@@ -318,9 +329,6 @@ do_genmove(int color, float pure_threat_value,
   move = PASS_MOVE;  
   *value = 0.0; 
   
-  /* experimental level adapter */
-  clock_adapt_level(&level, color);
-
   /* Prepare pattern matcher and reading code. */
   reset_engine();
 
@@ -341,7 +349,7 @@ do_genmove(int color, float pure_threat_value,
 
   /* Find out information about the worms and dragons. */
   start_timer(1);
-  examine_position(EXAMINE_ALL);
+  examine_position(EXAMINE_ALL, 0);
   time_report(1, "examine position", NO_MOVE, 1.0);
 
 
@@ -396,8 +404,7 @@ do_genmove(int color, float pure_threat_value,
   time_report(1, "generate move reasons", NO_MOVE, 1.0);
   
   /* Try to find empty corner moves. */
-  if (!limit_search)
-    fuseki(color);
+  fuseki(color);
   gg_assert(stackp == 0);
 
   /* Look for moves to break mirror play by the opponent. */
@@ -407,7 +414,7 @@ do_genmove(int color, float pure_threat_value,
    * dragon dangerous and change its status from DEAD to
    * UNKNOWN. Otherwise, pretend there is no thrashing dragon.
    */
-  if (!doing_scoring && !limit_search)
+  if (!doing_scoring)
     use_thrashing_dragon_heuristics
       = revise_thrashing_dragon(color, pessimistic_score, 5.0);
   
@@ -431,7 +438,7 @@ do_genmove(int color, float pure_threat_value,
 
 
   /* If the move value is 6 or lower, we look for endgame patterns too. */
-  if (*value <= 6.0 && !disable_endgame_patterns && !limit_search) {
+  if (*value <= 6.0 && !disable_endgame_patterns) {
     endgame_shapes(color);
     endgame(color);
     gg_assert(stackp == 0);
@@ -479,10 +486,10 @@ do_genmove(int color, float pure_threat_value,
    * opponent stones, or if the opponent is trying to live inside
    * our territory and we are clearly ahead, generate an aftermath move.
    */
-  if (move == PASS_MOVE && !doing_scoring) {
+  if (move == PASS_MOVE) {
     if (play_out_aftermath 
 	|| capture_all_dead 
-	|| (thrashing_dragon && pessimistic_score > 15.0))
+	|| (!doing_scoring && thrashing_dragon && pessimistic_score > 15.0))
       move = aftermath_genmove(color, 0, allowed_moves);
       
     /* If we're instructed to capture all dead opponent stones, generate
@@ -670,7 +677,7 @@ find_mirror_move(int *move, int color)
  * black.
  */
 static void
-compute_scores()
+compute_scores(int use_chinese_rules)
 {
   signed char safe_stones[BOARDMAX];
   float strength[BOARDMAX];
@@ -678,11 +685,11 @@ compute_scores()
   set_strength_data(WHITE, safe_stones, strength);
   compute_influence(EMPTY, safe_stones, strength, &move_influence,
       		    NO_MOVE, "White territory estimate");
-  white_score = influence_score(&move_influence);
+  white_score = influence_score(&move_influence, use_chinese_rules);
   set_strength_data(BLACK, safe_stones, strength);
   compute_influence(EMPTY, safe_stones, strength, &move_influence,
       		    NO_MOVE, "White territory estimate");
-  black_score = influence_score(&move_influence);
+  black_score = influence_score(&move_influence, use_chinese_rules);
 
   if (verbose || showscore) {
     if (white_score == black_score)
@@ -774,16 +781,14 @@ should_resign(int color, float optimistic_score, int move)
  *                Mark a limited search area                         *
 \*********************************************************************/
 
-/* Mark a limited search area. Only affects the engine if the
- * global variable limit_search is nonzero. In this case, genmove
- * will only return moves within the area marked by the array
- * search_mask.
- */
+/* Activate or deactivate search limit. */
+void
+set_limit_search(int value)
+{
+  limit_search = value;
+}
 
-static int search_mask[BOARDMAX];
-
-/* The following function marks a diamond of radius 6 with center pos.
- */
+/* The following function marks a diamond of radius 6 with center pos.  */
 
 void
 set_search_diamond(int pos)
@@ -815,7 +820,7 @@ reset_search_mask()
 /* marks a single vertex */
 
 void
-set_search_limit(int pos, int value)
+set_search_mask(int pos, int value)
 {
   search_mask[pos] = value;
 }
