@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 and 2006       *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -85,17 +85,9 @@ const char *result_to_string(int result);
 
 
 void gnugo_clear_board(int boardsize);
-void gnugo_play_move(int i, int j, int color);
-int  gnugo_play_sgfnode(SGFNode *node, int to_move);
-int  gnugo_play_sgftree(SGFNode *root, int *until, SGFNode **curnode);
-
-int  gnugo_sethand(int handicap, SGFNode *root);
-
-float gnugo_genmove(int *i, int *j, int color, int *resign);
-
-int  gnugo_attack(int m, int n, int *i, int *j);
-int  gnugo_find_defense(int m, int n, int *i, int *j);
-
+void gnugo_play_move(int move, int color);
+int gnugo_play_sgfnode(SGFNode *node, int to_move);
+int gnugo_sethand(int desired_handicap, SGFNode *node);
 float gnugo_estimate_score(float *upper, float *lower);
 
 /* ================================================================ */
@@ -105,17 +97,13 @@ float gnugo_estimate_score(float *upper, float *lower);
 
 typedef struct {
   int handicap;
-
   int to_move;		/* whose move it currently is */
   SGFTree game_record;	/* Game record in sgf format. */
-
   int computer_player;	/* BLACK, WHITE, or EMPTY (used as BOTH) */
 } Gameinfo;
 
-void gameinfo_clear(Gameinfo *ginfo, int boardsize, float komi);
+void gameinfo_clear(Gameinfo *ginfo);
 void gameinfo_print(Gameinfo *ginfo);
-void gameinfo_load_sgfheader(Gameinfo *gameinfo, SGFNode *head);
-void gameinfo_play_move(Gameinfo *ginfo, int i, int j, int color);
 int gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFTree *tree,
 			      const char *untilstr, int orientation);
 int gameinfo_play_sgftree(Gameinfo *gameinfo, SGFTree *tree,
@@ -148,6 +136,7 @@ extern int output_flags;       /* amount of output to outfile */
 
 /* debug flag bits */
 /* NOTE : can specify -d0x... */
+/* Please keep this list in sync with the DEBUG_FLAGS string below. */
 #define DEBUG_INFLUENCE             0x0001
 #define DEBUG_EYES                  0x0002
 #define DEBUG_OWL                   0x0004
@@ -175,39 +164,45 @@ extern int output_flags;       /* amount of output to outfile */
 #define DEBUG_ORACLE_STREAM         0x1000000
 #define DEBUG_LARGE_SCALE           0x1000000
 #define DEBUG_SPLIT_OWL             0x2000000
+#define DEBUG_TIME                  0x4000000
 
-/* hash flag bits 
- *
- * Regarding HASH_DEFAULT:
- * Hashing all functions saves time, but wastes table space, which is
- * bad when the reading is complicated. HASH_DEFAULT is a compromise. 
- *
- * FIXME: This is no longer true with the newer transposition table
- *        and its fancy replacement scheme. We can now hash everything.
- */
 
-#define HASH_FIND_DEFENSE 0x0001  /* NOTE : can specify -d0x... */
-#define HASH_ATTACK       0x0020
-#define HASH_OWL_ATTACK   0x0100
-#define HASH_OWL_DEFEND   0x0200
-#define HASH_SEMEAI       0x0400
-#define HASH_CONNECT      0x0800
-#define HASH_DISCONNECT   0x1000
-#define HASH_BREAK_IN	  0x2000
-#define HASH_BLOCK_OFF	  0x4000
-#define HASH_NOTHING      0
-#define HASH_ALL          0xffff
-#define HASH_DEFAULT      (HASH_ATTACK | HASH_FIND_DEFENSE\
-			   | HASH_OWL_ATTACK | HASH_OWL_DEFEND | HASH_SEMEAI\
-                           | HASH_CONNECT | HASH_DISCONNECT\
-			   | HASH_BREAK_IN | HASH_BLOCK_OFF)
+#define DEBUG_FLAGS "\
+DEBUG_INFLUENCE             0x0001\n\
+DEBUG_EYES                  0x0002\n\
+DEBUG_OWL                   0x0004\n\
+DEBUG_ESCAPE                0x0008\n\
+DEBUG_MATCHER               0x0010\n\
+DEBUG_DRAGONS               0x0020\n\
+DEBUG_SEMEAI                0x0040\n\
+DEBUG_LOADSGF               0x0080\n\
+DEBUG_HELPER                0x0100\n\
+DEBUG_READING               0x0200\n\
+DEBUG_WORMS                 0x0400\n\
+DEBUG_MOVE_REASONS          0x0800\n\
+DEBUG_OWL_PERFORMANCE       0x1000\n\
+DEBUG_BREAKIN               0x2000\n\
+DEBUG_FILLLIB               0x4000\n\
+DEBUG_READING_PERFORMANCE   0x8000\n\
+DEBUG_SCORING               0x010000\n\
+DEBUG_AFTERMATH             0x020000\n\
+DEBUG_ATARI_ATARI           0x040000\n\
+DEBUG_READING_CACHE         0x080000\n\
+DEBUG_TERRITORY             0x100000\n\
+DEBUG_PERSISTENT_CACHE      0x200000\n\
+DEBUG_TOP_MOVES             0x400000\n\
+DEBUG_MISCELLANEOUS         0x800000\n\
+DEBUG_ORACLE_STREAM         0x1000000\n\
+DEBUG_LARGE_SCALE           0x1000000\n\
+DEBUG_SPLIT_OWL             0x2000000\n\
+DEBUG_TIME                  0x4000000\n\
+"
+
 
 extern int debug;		/* debug flags */
-extern int hashflags;		/* hash flags */
 extern int fusekidb;            /* use fuseki database */
 extern int disable_fuseki;      /* do not generate fuseki moves */
 extern int josekidb;            /* use joseki database */
-extern int level;		/* controls depth of reading */
 extern int semeai_variations;   /* max variations considered reading semeai */
 extern int showtime;		/* print genmove time */
 extern int showscore;		/* print score */
@@ -238,10 +233,12 @@ extern int mandated_owl_distrust_depth;
 extern int mandated_owl_branch_depth;
 extern int mandated_owl_reading_depth;
 extern int mandated_owl_node_limit; 
+extern int mandated_semeai_node_limit; 
+
+extern int autolevel_on;
 
 extern float potential_moves[BOARDMAX];
 
-extern int limit_search;  /* limit move search to a portion of the board */
 extern int oracle_exists; /* oracle is available for consultation        */
 extern int metamachine;   /* use metamachine_genmove                     */
 
@@ -302,7 +299,7 @@ int DEBUG_func(int level, const char *fmt, ...);
 #define EXAMINE_ALL                 99
 
 void reset_engine(void);
-void examine_position(int how_much);
+void examine_position(int how_much, int aftermath_play);
 void silent_examine_position(int how_much);
 
 
@@ -318,7 +315,6 @@ void reset_reading_node_counter(void);
 int get_reading_node_counter(void);
 void reset_connection_node_counter(void);
 int get_connection_node_counter(void);
-void change_owl_node_limit(int new_limit, int *old_value);
 
 
 
@@ -335,7 +331,7 @@ int genmove(int color, float *value, int *resign);
 int genmove_conservative(int color, float *value);
 
 /* Play through the aftermath. */
-float aftermath_compute_score(int color, float komi, SGFTree *tree);
+float aftermath_compute_score(int color, SGFTree *tree);
 
 /* Basic information gathering. */
 /* worm.c */
@@ -361,6 +357,7 @@ void sgffile_output(SGFTree *tree);
 void sgffile_printsgf(int color_to_play, const char *filename);
 void sgffile_printboard(SGFTree *tree);
 void sgffile_recordboard(SGFNode *node);
+int get_sgfmove(SGFProperty *property);
 
 /* sgfdecide.c */
 void decide_string(int pos);
