@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 and 2006       *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -627,6 +627,55 @@ play_connect_n(int color, int do_connect, int num_moves, ...)
 }
 
 
+/* The function play_lib_n() plays a sequence of moves, alternating
+ * between the players and starting with color. After having played
+ * through the sequence, the last coordinate gives a target for liberty
+ * counting. The number of liberties is returned.
+ *
+ * If only one move is to be played and that stone is the target,
+ * accuratelib (or approxlib if appropriate) is more efficient.
+ */
+
+int 
+play_lib_n(int color, int num_moves, ...)
+{
+  va_list ap;
+  int mcolor = color;
+  int libs = 0;
+  int i;
+  int played_moves = 0;
+  int apos;
+  int ypos;
+
+  va_start(ap, num_moves);
+
+  /* Do all the moves with alternating colors. */
+  for (i = 0; i < num_moves; i++) {
+    apos = va_arg(ap, int);
+
+    if (apos != NO_MOVE
+	&& (trymove(apos, mcolor, "play_connect_n", NO_MOVE)
+	    || tryko(apos, mcolor, "play_connect_n")))
+      played_moves++;
+    mcolor = OTHER_COLOR(mcolor);
+  }
+
+  /* Now do the real work. */
+  ypos = va_arg(ap, int);
+  if (board[ypos] == EMPTY)
+    libs = 0;
+  else
+    libs = countlib(ypos);
+  
+  /* Pop all the moves we could successfully play. */
+  for (i = 0; i < played_moves; i++)
+    popgo();
+
+  va_end(ap);
+  return libs;
+}
+
+
 
 /* 
  * It is assumed in reading a ladder if stackp >= depth that
@@ -774,7 +823,7 @@ set_depth_values(int level, int report_levels)
 
   connect_depth         = gg_max(2, CONNECT_DEPTH  + 2 * depth_level);
   connect_depth2        = gg_max(2, CONNECT_DEPTH2 + 2 * depth_level);
-  connection_node_limit = CONNECT_NODE_LIMIT * pow(1.5, depth_level);
+  connection_node_limit = CONNECT_NODE_LIMIT * pow(1.45, depth_level);
   breakin_depth 	= gg_max(2, BREAKIN_DEPTH + 2 * depth_level);
   breakin_node_limit 	= BREAKIN_NODE_LIMIT * pow(1.5, depth_level);
 
@@ -804,6 +853,8 @@ set_depth_values(int level, int report_levels)
     owl_reading_depth = mandated_owl_reading_depth;
   if (mandated_owl_node_limit != -1)
     owl_node_limit = mandated_owl_node_limit;
+  if (mandated_semeai_node_limit != -1)
+    semeai_node_limit = mandated_semeai_node_limit;
 
   depth_offset = 0;
   
@@ -841,6 +892,8 @@ breakin_node_limit: %d\n\n",
 }
 
 
+static int depth_modification = 0;
+
 /*
  * Modify the various tactical reading depth parameters. This is
  * typically used to avoid horizon effects. By temporarily increasing
@@ -863,6 +916,7 @@ modify_depth_values(int n)
   ko_depth           += n;
   breakin_depth	     += n;
   depth_offset       += n;
+  depth_modification += n;
 }
 
 void
@@ -877,58 +931,12 @@ decrease_depth_values(void)
   modify_depth_values(-1);
 }
 
-/* These functions allow more drastic temporary modifications of the
- * depth values. Typical use is to turn certain depth values way down
- * for reading where speed is more important than accuracy, e.g. for
- * the influence function.
- */
-
-static int save_depth;
-static int save_backfill_depth;
-static int save_backfill2_depth;
-static int save_break_chain_depth;
-static int save_superstring_depth;
-static int save_branch_depth;
-static int save_fourlib_depth;
-static int save_ko_depth;
-
-/* Currently this function is never called. */
-
-void
-set_temporary_depth_values(int d, int b, int b2, int bc,
-			   int ss, int br, int f, int k)
+int
+get_depth_modification(void)
 {
-  save_depth             = depth;
-  save_backfill_depth    = backfill_depth;
-  save_backfill2_depth   = backfill2_depth;
-  save_break_chain_depth = break_chain_depth;
-  save_superstring_depth = superstring_depth;
-  save_branch_depth      = branch_depth;
-  save_fourlib_depth     = fourlib_depth;
-  save_ko_depth          = ko_depth;
-
-  depth             = d;
-  backfill_depth    = b;
-  backfill2_depth   = b2;
-  break_chain_depth = bc;
-  superstring_depth = ss;
-  branch_depth      = br;
-  fourlib_depth     = f;
-  ko_depth          = k;
+  return depth_modification;
 }
 
-void
-restore_depth_values()
-{
-  depth             = save_depth;
-  backfill_depth    = save_backfill_depth;
-  backfill2_depth   = save_backfill2_depth;
-  break_chain_depth = save_break_chain_depth;
-  superstring_depth = save_superstring_depth;
-  branch_depth      = save_branch_depth;
-  fourlib_depth     = save_fourlib_depth;
-  ko_depth          = save_ko_depth;
-}
 
 /*******************
  * Detect blunders *
@@ -1437,6 +1445,26 @@ send_two_return_one(int move, int color)
 }
 
 
+/* Returns true if a move by (color) plays into a snapback. */
+int
+playing_into_snapback(int move, int color)
+{
+  int libs[2];
+  int k;
+  
+  if (approxlib(move, color, 1, NULL) != 0
+      || accuratelib(move, color, 2, libs) != 1)
+    return 0;
+
+  for (k = 0; k < 4; k++)
+    if (board[move + delta[k]] == color
+	&& adjacent_strings(libs[0], move + delta[k]))
+      return 1;
+
+  return 0;
+}
+
+
 /* Score the game and determine the winner */
 
 void
@@ -1459,7 +1487,7 @@ who_wins(int color, FILE *outfile)
   /* Use the aftermath code to compute the final score. (Slower but
    * more reliable.) 
    */
-  result = aftermath_compute_score(color, komi);
+  result = aftermath_compute_score(color, NULL);
   if (result > 0.0)
     winner = WHITE;
   else {
