@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 and 2006       *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -260,7 +260,6 @@ static struct gg_option const long_options[] =
   {"capture-all-dead",   no_argument,   0, OPT_CAPTURE_ALL_DEAD},
   {"play-out-aftermath", no_argument,   0, OPT_PLAY_OUT_AFTERMATH},
   {"cache-size",     required_argument, 0, 'M'},
-  {"hash",           required_argument, 0, 'H'},
   {"worms",          no_argument,       0, 'w'},
   {"moyo",           required_argument, 0, 'm'},
   {"benchmark",      required_argument, 0, 'b'},
@@ -337,15 +336,12 @@ main(int argc, char *argv[])
   int seed = 0;
   int seed_specified = 0;
 
-  /* Default parameters for clock and auto level systems. */
-  clock_init(3600, 0, 0);      /* One hour sudden death. */
-
   sgftree_clear(&sgftree);
-  gameinfo_clear(&gameinfo, board_size, komi);
+  gameinfo_clear(&gameinfo);
   
   /* Weed through all of the command line options. */
   while ((i = gg_getopt_long(argc, argv, 
-                            "-ab:B:d:D:EF:gh::H:K:l:L:M:m:o:O:p:r:fsStTvw",
+                            "-ab:B:d:D:EF:gh::K:l:L:M:m:o:O:p:r:fsStTvw",
 			     long_options, NULL)) != EOF)
     {
       switch (i) {
@@ -365,11 +361,7 @@ main(int argc, char *argv[])
 	break;
       case 'd': debug ^= strtol(gg_optarg, NULL, 0);  /* allows 0x... */ break;
       case 'D': mandated_depth = atoi(gg_optarg); break;
-
       case 'M': memory = atof(gg_optarg); break; /* floating point number */
-      case 'H': hashflags = strtol(gg_optarg, NULL, 0);  /* allows 0x... */ 
-	break;
-
       case 'E': printboard = 2; break;
       case 'B': mandated_backfill_depth = atoi(gg_optarg); break;
       case 'F': mandated_fourlib_depth = atoi(gg_optarg); break;
@@ -472,8 +464,11 @@ main(int argc, char *argv[])
 		"Owl node limit: %d\n", OWL_NODE_LIMIT);
 	fprintf(stdout,
 		"Semeai node limit: %d\n", SEMEAI_NODE_LIMIT);
-	fprintf(stdout,
-		"Cache size: %d MB\n", DEFAULT_MEMORY);
+	if (DEFAULT_MEMORY == -1)
+	  fprintf(stdout, "Cache size: %d MB (special default value)\n",
+		  DEFAULT_MEMORY);
+	else
+	  fprintf(stdout, "Cache size: %d MB\n", DEFAULT_MEMORY);
 
 	return EXIT_SUCCESS;
 	break;
@@ -488,14 +483,14 @@ main(int argc, char *argv[])
 	
       case OPT_HANDICAPSTONES:
 	{
-	  int handicap = atoi(gg_optarg);
+	  int requested_handicap = atoi(gg_optarg);
 	  
-	  if (handicap < 0 || handicap > MAX_HANDICAP) {
-	    fprintf(stderr, "Unsupported handicap: %d.\n", handicap);
+	  if (requested_handicap < 0 || requested_handicap > MAX_HANDICAP) {
+	    fprintf(stderr, "Unsupported handicap: %d.\n", requested_handicap);
 	    fprintf(stderr, "Try `gnugo --help' for more information.\n");
 	    exit(EXIT_FAILURE);
 	  }
-	  gameinfo.handicap = handicap;
+	  gameinfo.handicap = requested_handicap;
 	}
         break;
       
@@ -504,7 +499,11 @@ main(int argc, char *argv[])
 	  int boardsize = atoi(gg_optarg);
 	  
 	  if (boardsize < MIN_BOARD || boardsize > MAX_BOARD) {
-	    fprintf(stderr, "Unsupported board size: %d.\n", boardsize);
+	    fprintf(stderr, "Unsupported board size: %d. ", boardsize);
+	    if (boardsize < MIN_BOARD)
+	      fprintf(stderr, "Min size is %d.\n", MIN_BOARD);
+	    else
+	      fprintf(stderr, "Max size is %d.\n", MAX_BOARD);
 	    fprintf(stderr, "Try `gnugo --help' for more information.\n");
 	    exit(EXIT_FAILURE);
 	  }
@@ -545,7 +544,7 @@ main(int argc, char *argv[])
 	break;
 
       case OPT_SEMEAI_NODE_LIMIT:
-	semeai_node_limit = atoi(gg_optarg);
+	mandated_semeai_node_limit = atoi(gg_optarg);
 	break;
 
       case OPT_EXPERIMENTAL_CONNECTIONS:
@@ -780,19 +779,15 @@ main(int argc, char *argv[])
 	break;
 	
       case OPT_LEVEL:
-	level = atoi(gg_optarg);
-	if (level > max_level)
-	  max_level = level;
-	if (level < min_level)
-	  min_level = level;
+	set_level(atoi(gg_optarg));
 	break;
 
       case OPT_MIN_LEVEL:
-	min_level = atoi(gg_optarg);
+	set_min_level(atoi(gg_optarg));
 	break;
 
       case OPT_MAX_LEVEL:
-	max_level = atoi(gg_optarg);
+	set_max_level(atoi(gg_optarg));
 	break;
 
       case OPT_LIMIT_SEARCH:
@@ -807,25 +802,19 @@ main(int argc, char *argv[])
 	break;
 
       case OPT_CLOCK_TIME:
-
-	clock_init(atoi(gg_optarg), -1, -1);
-	clock_enable();
+	clock_settings(atoi(gg_optarg), -1, -1);
 	break;
 
       case OPT_CLOCK_BYO_TIME: 
-	clock_init(-1, atoi(gg_optarg), -1);
-	clock_enable();
+	clock_settings(-1, atoi(gg_optarg), -1);
 	break;
 
       case OPT_CLOCK_BYO_PERIOD:
-	clock_init(-1, -1, atoi(gg_optarg));
-	clock_enable();
+	clock_settings(-1, -1, atoi(gg_optarg));
 	break;
 
       case OPT_AUTOLEVEL:
-	clock_init(-1, -1, -1);
-	clock_enable();
-	clock_enable_autolevel();
+	autolevel_on = 1;
 	break;
 	
       case OPT_DEBUG_INFLUENCE:
@@ -928,7 +917,7 @@ main(int argc, char *argv[])
       case OPT_PRINT_LEVELS:
 	{
 	  int lev;
-	  for (lev = 10; lev > 0; lev--)
+	  for (lev = 12; lev >= 0; lev--)
 	    set_depth_values(lev, 1);
 	}
 	return EXIT_SUCCESS;
@@ -961,7 +950,7 @@ main(int argc, char *argv[])
   }
   else
   /* Initialize and empty sgf tree if there was no infile. */
-    sgftreeCreateHeaderNode(&sgftree, board_size, komi);
+    sgftreeCreateHeaderNode(&sgftree, board_size, komi, handicap);
 
   /* Set the game_record to be identical to the loaded one or the
    * newly created empty sgf tree.
@@ -1024,7 +1013,7 @@ main(int argc, char *argv[])
       fprintf(stderr, "You must use -l infile with replay mode.\n");
       exit(EXIT_FAILURE);
     }
-    play_replay(&gameinfo, replay_color);
+    play_replay(&sgftree, replay_color);
     break;
     
   case MODE_LOAD_AND_ANALYZE:
@@ -1364,7 +1353,14 @@ main(int argc, char *argv[])
       show_version();
       show_copyright();
     }
-  
+
+#if ORACLE
+    if (metamachine) {
+      summon_oracle();
+      oracle_loadsgf(infilename, untilstring);
+    }
+#endif
+
     play_ascii(&sgftree, &gameinfo, infilename, untilstring);
     break;
   }
@@ -1372,7 +1368,6 @@ main(int argc, char *argv[])
   if (profile_patterns)
     report_pattern_profiling();
 
-  clock_report_autolevel(NULL, gameinfo.computer_player);
   sgfFreeNode(sgftree.root); 
 
   return 0;
@@ -1402,7 +1397,8 @@ show_version(void)
 
 #define USAGE "\n\
 Usage: gnugo [-opts]\n\
-Examples:\
+\n\
+Examples:\n\
   gnugo --mode gtp --level 5\n\
          To play against gnugo in level 5 from a GTP client\n\
   gnugo --mode ascii -l game.sgf -L 123\n\
@@ -1415,29 +1411,59 @@ Main Options:\n\
                          or 'gtp'). Default is ASCII.\n\
                          If no terminal is detected GMP (Go Modem Protocol)\n\
                          will be assumed.\n\
-       --quiet           Don't print copyright and informational messages\n\
+       --quiet  --silent Don't print copyright and informational messages\n\
        --level <amount>  strength (default %d)\n\
        --never-resign    Forbid GNU Go to resign\n\
        --resign-allowed  Allow resignation (default)\n\
-       --gtp-input <file>Read gtp commands from file instead of stdin\n\
-       --gtp-connect [HOST:]PORT\n\
-                         Connect to given host (127.0.0.1 if omitted) and port\n\
-                         and receive GTP commands on the established connection\n\
-       --gtp-listen [HOST:]PORT\n\
-                         Wait for the first TCP/IP connection on the given port\n\
-                         (if HOST is specified, only to that host)\n\
    -l, --infile <file>   Load name sgf file\n\
    -L, --until <move>    Stop loading just before move is played. <move>\n\
                          can be the move number or location (eg L10).\n\
    -o, --outfile <file>  Write sgf output to file\n\
-   --printsgf <file>     Write position as a diagram to file (use with -l)\n\
+       --printsgf <file>     Write position as a diagram to file (use with -l)\n\
+\n\
+Scoring:\n\
+   --score estimate      estimate score at loaded position\n\
+   --score finish        generate moves to finish game, then score\n\
+   --score aftermath     generate moves to finish, use best algorithm\n\
+\n\
 "
 
 #define USAGE1 "\
+Game Options:\n\
+Used with --mode ascii (or other modes for non-interactive settings)\n\
+   --boardsize num   Set the board size to use (%d--%d)\n\
+   --color <color>   Choose your color ('black' or 'white')\n\
+   --handicap <num>  Set the number of handicap stones (0--%d)\n\
+   --komi <num>      Set the komi\n\
+   --clock <sec>     Initialize the timer.\n\
+   --byo-time <sec>  Initialize the byo-yomi timer.\n\
+   --byo-period <stones>  Initialize the byo-yomi period.\n\
+\n\
+   --japanese-rules     (default)\n\
+   --chinese-rules\n\
+   --allow-suicide\n\
+\n\
+   --play-out-aftermath\n\
+   --capture-all-dead\n\
+\n\
    --min-level <amount>         minimum level for adjustment schemes\n\
    --max-level <amount>         maximum level for adjustment schemes\n\
    --autolevel                  adapt gnugo level during game to respect\n\
                                 the time specified by --clock <sec>.\n\
+\n\
+Connection options\n\
+   --gtp-input <file>Read gtp commands from file instead of stdin\n\
+   --gtp-connect [HOST:]PORT\n\
+                     Connect to given host (127.0.0.1 if omitted) and port\n\
+                     and receive GTP commands on the established connection\n\
+   --gtp-listen [HOST:]PORT\n\
+                     Wait for the first TCP/IP connection on the given port\n\
+                     (if HOST is specified, only to that host)\n\
+   --gtp-version\n\
+\n\
+"
+
+#define USAGE2 "\
 Experimental options:\n\
    --with-break-in         use the break-in code (on at level 10 by default)\n\
    --without-break-in      do not use the break-in code\n\
@@ -1450,26 +1476,16 @@ Experimental options:\n\
    --nojosekidb            turn off joseki database\n\
    --mirror                try to play mirror go\n\
    --mirror-limit <n>      stop mirroring when n stones on board\n\n\
-Scoring:\n\
-   --score estimate        estimate score at loaded position\n\
-   --score finish          generate moves to finish game, then score\n\
-   --score aftermath       generate moves to finish, use best algorithm\n\
-   --score aftermath --capture-all-dead --chinese-rules   Tromp-Taylor score\n\
+   --alternate-connections\n\
+   --experimental-connections\n\
+   --experimental-owl-ext\n\
+   --experimental-semeai\n\
+   --standard-connections\n\
+   --standard-semeai\n\
+   --oracle                Read the documentation\n\
 \n\
 Cache size (higher=more memory usage, faster unless swapping occurs):\n\
-   -M, --cache-size <megabytes>  RAM cache for hashing (default %4.1f Mb)\n\
-\n\
-"
-
-#define USAGE2 "\
-Game Options: (--mode ascii)\n\
-       --boardsize num   Set the board size to use (%d--%d)\n\
-       --color <color>   Choose your color ('black' or 'white')\n\
-       --handicap <num>  Set the number of handicap stones (0--%d)\n\
-       --komi <num>      Set the komi\n\
-       --clock <sec>     Initialize the timer.\n\
-       --byo-time <sec>  Initialize the byo-yomi timer.\n\
-       --byo-period <stones>  Initialize the byo-yomi period.\n\
+   -M, --cache-size <megabytes>  RAM cache for read results (default %4.1f Mb)\n\
 \n\
 Informative Output:\n\
    -v, --version         Display the version and copyright of GNU Go\n\
@@ -1480,10 +1496,10 @@ Informative Output:\n\
 \n\
 "
 
-
 #define COPYRIGHT \
-"Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005 by the Free Software\n\
-Foundation, Inc. See http://www.gnu.org/software/gnugo/ or contact\n\
+"Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 and 2006\n\
+by the Free Software Foundation, Inc.\n\
+See http://www.gnu.org/software/gnugo/ or contact\n\
 gnugo@gnu.org for information about GNU Go. GNU Go comes with NO WARRANTY to\n\
 the extent permitted by law. This program is free software; you can\n\
 redistribute it and/or modify it under the terms of the GNU General Public\n\
@@ -1505,10 +1521,9 @@ Debugging Options:\n\
    -E, --printeyes               colored display of eye spaces\n\
    -d, --debug <flags>           debugging output (see next item for bits)\n\
        --debug-flags             print the debug flags for previous item\n\
-   -H, --hash <level>            hash (see gnugo.h for bits)\n\
    -w, --worms                   print worm and dragon data and move reasons\n\
    -m, --moyo <level>            moyo debugging, show moyo board\n\
-       --debuginfluence <move>   print influence map after making a move\n\
+       --debug-influence <move>   print influence map after making a move\n\
    -b, --benchmark num           benchmarking mode - can be used with -l\n\
    -S, --statistics              print statistics (for debugging purposes)\n\n\
        --profile-patterns        print statistics for pattern usage\n\
@@ -1521,10 +1536,16 @@ Debugging Options:\n\
        --showscore               print estimated score\n\
    -r, --seed number             set random number seed\n\
        --gtp-dump-commands <file>dump commands received in GTP mode\n\
+       --gtp-initial-orientation\n\
+       --orientation\n\
+\n\
 "
 
 #define USAGE_DEBUG2 "\
 Options affecting depth settings and playing strength:\n\
+   --print-levels        shows all this values for levels 12 to 0\n\
+\n\
+   Default values for the default level (%d):\n\
    -D, --depth <depth>          deep reading cutoff (default %d)\n\
    -B, --backfill-depth <depth> deep reading cutoff (default %d)\n\
    -F, --fourlib-depth <depth>  deep reading cutoff (default %d)\n\
@@ -1538,44 +1559,25 @@ Options affecting depth settings and playing strength:\n\
    --owl-branch <depth>         owl branching depth (default %d)\n\
    --owl-reading <depth>        owl reading depth (default %d)\n\
    --owl-node-limit <limit>     max nodes for owl reading (default %d)\n\
+   --semeai-node-limit <limit>  max nodes for semeai reading (default %d)\n\
+\n\
 Options providing detailed reading results etc.:\n\
    --decide-string <string>     can this string live? (try with -o)\n\
    --decide-connection <str/str> can these strings connect? (try with -o)\n\
    --decide-dragon <dragon>     can this dragon live? (try with -o or -t)\n\
+   --decide-dragon-data\n\
+   --decide-owl\n\
    --decide-position            evaluate all dragons (try with -o or -t)\n\
    --decide-eye <string>        evaluate the eye\n\
    --decide-combination         search for combination attack (try with -o)\n\
-   --genmove <color>            generate a move for color\n\
+   --decide-oracle\n\
+   --decide-semeai\n\
+   --decide-tactical-semeai\n\
+   --decide-surrounded\n\
+   --limit-search\n\
+\n\
 "
 
-#define DEBUG_FLAGS "\
-DEBUG_INFLUENCE             0x0001\n\
-DEBUG_EYES                  0x0002\n\
-DEBUG_OWL                   0x0004\n\
-DEBUG_ESCAPE                0x0008\n\
-DEBUG_MATCHER               0x0010\n\
-DEBUG_DRAGONS               0x0020\n\
-DEBUG_SEMEAI                0x0040\n\
-DEBUG_LOADSGF               0x0080\n\
-DEBUG_HELPER                0x0100\n\
-DEBUG_READING               0x0200\n\
-DEBUG_WORMS                 0x0400\n\
-DEBUG_MOVE_REASONS          0x0800\n\
-DEBUG_OWL_PERFORMANCE       0x1000\n\
-DEBUG_BREAKIN               0x2000\n\
-DEBUG_FILLLIB               0x4000\n\
-DEBUG_READING_PERFORMANCE   0x8000\n\
-DEBUG_SCORING               0x010000\n\
-DEBUG_AFTERMATH             0x020000\n\
-DEBUG_ATARI_ATARI           0x040000\n\
-DEBUG_READING_CACHE         0x080000\n\
-DEBUG_TERRITORY             0x100000\n\
-DEBUG_OWL_PERSISTENT_CACHE  0X200000\n\
-DEBUG_TOP_MOVES             0x400000\n\
-DEBUG_MISCELLANEOUS         0x800000\n\
-DEBUG_ORACLE_STREAM         0x1000000\n\
-DEBUG_LARGE_SCALE           0x1000000\n\
-"
 
 /*
  * Since the maximum string length is 2048 bytes in VC++ we
@@ -1584,21 +1586,22 @@ DEBUG_LARGE_SCALE           0x1000000\n\
 static void
 show_help(void)
 {
-  set_depth_values(DEFAULT_LEVEL, 0);
-  printf(USAGE, level);
-  printf(USAGE1, (float) DEFAULT_MEMORY);
-  printf(USAGE2, MIN_BOARD, MAX_BOARD, MAX_HANDICAP);
+  printf(USAGE, DEFAULT_LEVEL);
+  printf(USAGE1, MIN_BOARD, MAX_BOARD, MAX_HANDICAP);
+  printf(USAGE2, DEFAULT_MEMORY <= 0 ? reading_cache_default_size() :
+	 (float) DEFAULT_MEMORY);
 }
 
 
 static void
 show_debug_help(void)
 {
+  set_depth_values(DEFAULT_LEVEL,0);
   printf(USAGE_DEBUG USAGE_DEBUG2, 
-	 depth, backfill_depth, fourlib_depth, ko_depth, branch_depth,
+	 DEFAULT_LEVEL, depth, backfill_depth, fourlib_depth, ko_depth, branch_depth,
 	 backfill2_depth, break_chain_depth, superstring_depth, aa_depth, 
 	 owl_distrust_depth, owl_branch_depth,
-	 owl_reading_depth, owl_node_limit);
+	 owl_reading_depth, owl_node_limit, semeai_node_limit);
 }
 
 static void 
