@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 and 2006       *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -27,7 +27,7 @@
 #include <string.h>
 #include <ctype.h>
 
-#ifdef READLINE
+#if READLINE
 #include <readline/readline.h>
 #include <readline/history.h>
 #endif
@@ -54,6 +54,8 @@ static int showdead = 0;
 static SGFTree sgftree;
 static int resignation_allowed;
 
+static int clock_on = 0;
+
 /* Unreasonable score used to detect missing information. */
 #define NO_SCORE 4711
 /* Keep track of the score estimated before the last computer move. */
@@ -65,7 +67,7 @@ static void ascii_count(Gameinfo *gameinfo);
 static void showcapture(char *line);
 static void showdefense(char *line);
 static void ascii_goto(Gameinfo *gameinfo, char *line);
-static void ascii_free_handicap(Gameinfo *gameinfo, char *handicap);
+static void ascii_free_handicap(Gameinfo *gameinfo, char *handicap_string);
 
 /* If sgf game info is written can't reset parameters like handicap, etc. */
 static int sgf_initialized;
@@ -159,7 +161,7 @@ set_handicap_spots(int boardsize)
       /* ...and the middle one. */
       hspots[boardsize/2][boardsize/2] = '+';
     }
-    else {
+    else if (boardsize > 12) {
       /* Place the outer 4... */
       hspots[3][3] = '+';
       hspots[boardsize-4][3] = '+';
@@ -296,6 +298,11 @@ ascii_showboard(void)
   fflush(stdout);
   printf("%s\n\n", letterbar);
   fflush(stdout);
+
+  if (clock_on) {
+    clock_print(WHITE);
+    clock_print(BLACK);
+  }
   
 }  /* end ascii_showboard */
 
@@ -440,8 +447,7 @@ init_sgf(Gameinfo *ginfo)
   sgf_initialized = 1;
 
   sgf_write_header(sgftree.root, 1, get_random_seed(), komi,
-      		   level, chinese_rules);
-  sgfOverwritePropertyInt(sgftree.root, "HA", ginfo->handicap);
+      		   ginfo->handicap, get_level(), chinese_rules);
   if (ginfo->handicap > 0)
     sgffile_recordboard(sgftree.root);
 }
@@ -454,7 +460,7 @@ init_sgf(Gameinfo *ginfo)
 static int
 computer_move(Gameinfo *gameinfo, int *passes)
 {
-  int i, j;
+  int move;
   float move_value;
   int resign;
   int resignation_declined = 0;
@@ -463,7 +469,9 @@ computer_move(Gameinfo *gameinfo, int *passes)
   init_sgf(gameinfo);
 
   /* Generate computer move. */
-  move_value = gnugo_genmove(&i, &j, gameinfo->to_move, &resign);
+  if (autolevel_on)
+    adjust_level_offset(gameinfo->to_move);
+  move = genmove(gameinfo->to_move, &move_value, &resign);
   if (resignation_allowed && resign) {
     int state = ascii_endgame(gameinfo, 2);
     if (state != -1)
@@ -479,16 +487,16 @@ computer_move(Gameinfo *gameinfo, int *passes)
     current_score_estimate = (int) ((lower_bound + upper_bound) / 2.0);
   }
   
-  mprintf("%s(%d): %m\n", color_to_string(gameinfo->to_move),
-	  movenum+1, i, j);
-  if (is_pass(POS(i, j)))
+  mprintf("%s(%d): %1m\n", color_to_string(gameinfo->to_move),
+	  movenum + 1, move);
+  if (is_pass(move))
     (*passes)++;
   else
     *passes = 0;
 
-  gnugo_play_move(i, j, gameinfo->to_move);
+  gnugo_play_move(move, gameinfo->to_move);
   sgffile_add_debuginfo(sgftree.lastnode, move_value);
-  sgftreeAddPlay(&sgftree, gameinfo->to_move, i, j);
+  sgftreeAddPlay(&sgftree, gameinfo->to_move, I(move), J(move));
   if (resignation_declined)
     sgftreeAddComment(&sgftree, "GNU Go resignation was declined");
   sgffile_output(&sgftree);
@@ -520,7 +528,7 @@ do_move(Gameinfo *gameinfo, char *command, int *passes, int force)
   *passes = 0;
   TRACE("\nyour move: %1m\n\n", move);
   init_sgf(gameinfo);
-  gnugo_play_move(I(move), J(move), gameinfo->to_move);
+  gnugo_play_move(move, gameinfo->to_move);
   sgffile_add_debuginfo(sgftree.lastnode, 0.0);
   sgftreeAddPlay(&sgftree, gameinfo->to_move, I(move), J(move));
   sgffile_output(&sgftree);
@@ -551,7 +559,7 @@ do_pass(Gameinfo *gameinfo, int *passes, int force)
 {
   (*passes)++;
   init_sgf(gameinfo);
-  gnugo_play_move(-1, -1, gameinfo->to_move);
+  gnugo_play_move(PASS_MOVE, gameinfo->to_move);
   sgffile_add_debuginfo(sgftree.lastnode, 0.0);
   sgftreeAddPlay(&sgftree, gameinfo->to_move, -1, -1);
   sgffile_output(&sgftree);
@@ -581,7 +589,6 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
   sgftree = *tree;
 
   if (filename) {
-    gameinfo_load_sgfheader(gameinfo, sgftree.root);
     gameinfo->to_move = gameinfo_play_sgftree(gameinfo, &sgftree, until);
     sgf_initialized = 1;
   }
@@ -618,6 +625,9 @@ do_play_ascii(Gameinfo *gameinfo)
   char *tmpstring;
   int state = 1;
 
+  if (have_time_settings())
+    clock_on = 1;
+
   while (state == 1) {
     state = 0;
 
@@ -642,7 +652,7 @@ do_play_ascii(Gameinfo *gameinfo)
       if (opt_showboard)
 	ascii_showboard();
 
-#ifndef READLINE
+#if !READLINE
       /* Print the prompt */
       mprintf("%s(%d): ", color_to_string(gameinfo->to_move), movenum + 1);
 
@@ -775,8 +785,8 @@ do_play_ascii(Gameinfo *gameinfo)
 	    printf("\nInvalid command syntax!\n");
 	    break;
 	  }
-	  level = num;
-	  printf("\nSet level to %d\n", level);
+	  set_level(num);
+	  printf("\nSet level to %d\n", num);
 	  break;
 
 	case DISPLAY:
@@ -887,7 +897,7 @@ do_play_ascii(Gameinfo *gameinfo)
 	  break;
 
 	case CMD_DEAD:
-	  examine_position(FULL_EXAMINE_DRAGONS);
+	  silent_examine_position(FULL_EXAMINE_DRAGONS);
 	  showdead = !showdead;
 	  break;
 
@@ -904,26 +914,26 @@ do_play_ascii(Gameinfo *gameinfo)
 	case CMD_SHOWMOYO:
 	  tmp = printmoyo;
 	  printmoyo = PRINTMOYO_MOYO;
-	  examine_position(EXAMINE_DRAGONS);
+	  silent_examine_position(EXAMINE_DRAGONS);
 	  printmoyo = tmp;
 	  break;
 
 	case CMD_SHOWTERRI:
 	  tmp = printmoyo;
 	  printmoyo = PRINTMOYO_TERRITORY;
-	  examine_position(EXAMINE_DRAGONS);
+	  silent_examine_position(EXAMINE_DRAGONS);
 	  printmoyo = tmp;
 	  break;
 
 	case CMD_SHOWAREA:
 	  tmp = printmoyo;
 	  printmoyo = PRINTMOYO_AREA;
-	  examine_position(EXAMINE_DRAGONS);
+	  silent_examine_position(EXAMINE_DRAGONS);
 	  printmoyo = tmp;
 	  break;
 
 	case CMD_SHOWDRAGONS:
-	  examine_position(EXAMINE_DRAGONS);
+	  silent_examine_position(EXAMINE_DRAGONS);
 	  showboard(1);
 	  break;
 
@@ -961,7 +971,6 @@ do_play_ascii(Gameinfo *gameinfo)
 	    }
             /* to avoid changing handicap etc. */
             sgf_initialized = 1;
-            gameinfo_load_sgfheader(gameinfo, sgftree.root);
             gameinfo_play_sgftree(gameinfo, &sgftree, NULL);
 	    sgfOverwritePropertyInt(sgftree.root, "HA", gameinfo->handicap);
 	  }
@@ -970,7 +979,7 @@ do_play_ascii(Gameinfo *gameinfo)
 	  break;
 
 	case CMD_LISTDRAGONS:
-	  examine_position(EXAMINE_DRAGONS);
+	  silent_examine_position(EXAMINE_DRAGONS);
 	  show_dragons();
 	  break;
 
@@ -982,7 +991,7 @@ do_play_ascii(Gameinfo *gameinfo)
 	if (passes >= 2)
 	  state = ascii_endgame(gameinfo, 0);
       }
-#ifdef READLINE
+#if READLINE
 	free(line_ptr);
 #endif
 
@@ -997,10 +1006,10 @@ do_play_ascii(Gameinfo *gameinfo)
     /* Free the sgf tree and prepare for a new game. */
     sgfFreeNode(sgftree.root);
     sgftree_clear(&sgftree);
-    sgftreeCreateHeaderNode(&sgftree, board_size, komi);
+    sgftreeCreateHeaderNode(&sgftree, board_size, komi, gameinfo->handicap);
     sgf_initialized = 0;
 
-    gameinfo_clear(gameinfo, board_size, komi);
+    gameinfo_clear(gameinfo);
   }
 }
 
@@ -1218,15 +1227,15 @@ ascii_goto(Gameinfo *gameinfo, char *line)
 
 
 static void
-ascii_free_handicap(Gameinfo *gameinfo, char *handicap)
+ascii_free_handicap(Gameinfo *gameinfo, char *handicap_string)
 {
   int handi;
   int i;
   char line[80];
   int stones[MAX_BOARD*MAX_BOARD];
 
-  if (sscanf(handicap, "%d", &handi) == 1) {
-    /* Gnu Go is to place handicap */
+  if (sscanf(handicap_string, "%d", &handi) == 1) {
+    /* GNU Go is to place handicap */
     if (handi < 0 || handi == 1) {
       printf("\nInvalid command syntax!\n");
       return;
